import minecraft_launcher_lib
import json
import os


def create_test_version_file(minecraft_directory: str):
    os.makedirs(os.path.join(minecraft_directory, "versions", "utilstest"))
    with open(os.path.join(minecraft_directory, "versions", "utilstest", "utilstest.json"), "w", encoding="utf-8") as f:
        data = {}
        data["id"] = "utilstest"
        data["type"] = "release"
        json.dump(data, f)


def test_get_minecraft_directory():
    assert isinstance(minecraft_launcher_lib.utils.get_minecraft_directory(), str)


def test_get_latest_version():
    latest_version = minecraft_launcher_lib.utils.get_latest_version()
    assert "release" in latest_version
    assert "snapshot" in latest_version


def test_get_version_list():
    version_list = minecraft_launcher_lib.utils.get_version_list()
    assert "type" in version_list[0]
    assert "id" in version_list[0]


def test_get_installed_versions(tmpdir):
    create_test_version_file(tmpdir)
    version_list = minecraft_launcher_lib.utils. get_installed_versions(str(tmpdir))
    assert version_list[0]["id"] == "utilstest"
    assert version_list[0]["type"] == "release"


def test_get_available_versions(tmpdir):
    create_test_version_file(tmpdir)
    version_list = minecraft_launcher_lib.utils.get_available_versions(str(tmpdir))
    assert "type" in version_list[0]
    assert "id" in version_list[0]


def test_get_java_executable():
    assert isinstance(minecraft_launcher_lib.utils.get_java_executable(), str)


def test_get_library_version():
    assert isinstance(minecraft_launcher_lib.utils.get_library_version(), str)


def test_generate_test_options():
    options = minecraft_launcher_lib.utils.generate_test_options()
    assert isinstance(options["username"], str)
    assert isinstance(options["uuid"], str)
    assert isinstance(options["token"], str)


def test_is_version_valid(tmpdir):
    create_test_version_file(tmpdir)
    assert minecraft_launcher_lib.utils.is_version_valid("1.16", str(tmpdir)) is True
    assert minecraft_launcher_lib.utils.is_version_valid("utilstest", str(tmpdir)) is True
    assert minecraft_launcher_lib.utils.is_version_valid("Test123", str(tmpdir)) is False



import minecraft_launcher_lib


def test_list_forge_versions():
    version_list = minecraft_launcher_lib.forge.list_forge_versions()
    assert isinstance(version_list[0], str)


def test_find_forge_version():
    assert isinstance(minecraft_launcher_lib.forge.find_forge_version("1.16.2"), str)
    assert minecraft_launcher_lib.forge.find_forge_version("Test123") is None


def test_is_forge_version_valid():
    assert minecraft_launcher_lib.forge.is_forge_version_valid("1.16.5-36.1.32") is True
    assert minecraft_launcher_lib.forge.is_forge_version_valid("Test123") is False


def test_supports_automatic_install():
    assert minecraft_launcher_lib.forge.supports_automatic_install("1.16.5-36.1.32") is True
    assert minecraft_launcher_lib.forge.supports_automatic_install("1.12-14.21.1.2443") is False



import minecraft_launcher_lib


def test_get_all_minecraft_versions():
    version_list = minecraft_launcher_lib.fabric.get_all_minecraft_versions()
    assert isinstance(version_list[0]["version"], str)
    assert isinstance(version_list[0]["stable"], bool)


def test_get_stable_minecraft_versions():
    version_list = minecraft_launcher_lib.fabric.get_stable_minecraft_versions()
    assert isinstance(version_list[0], str)


def test_get_latest_minecraft_version():
    assert isinstance(minecraft_launcher_lib.fabric.get_latest_minecraft_version(), str)


def test_get_latest_stable_minecraft_version():
    assert isinstance(minecraft_launcher_lib.fabric.get_latest_stable_minecraft_version(), str)


def test_is_minecraft_version_supported():
    assert minecraft_launcher_lib.fabric.is_minecraft_version_supported("1.16") is True
    assert minecraft_launcher_lib.fabric.is_minecraft_version_supported("1.0") is False


def test_get_all_loader_versions():
    version_list = minecraft_launcher_lib.fabric.get_all_loader_versions()
    assert isinstance(version_list[0]["separator"], str)
    assert isinstance(version_list[0]["build"], int)
    assert isinstance(version_list[0]["maven"], str)
    assert isinstance(version_list[0]["version"], str)
    assert isinstance(version_list[0]["stable"], bool)


def test_get_latest_loader_version():
    assert isinstance(minecraft_launcher_lib.fabric.get_latest_loader_version(), str)


def test_get_latest_installer_version():
    assert isinstance(minecraft_launcher_lib.fabric.get_latest_installer_version(), str)



import minecraft_launcher_lib
import pytest


def test_extract_natives_invalid_version(tmpdir):
    # Checks if the VersionNotFound exception raised
    with pytest.raises(minecraft_launcher_lib.exceptions.VersionNotFound):
        minecraft_launcher_lib.natives.extract_natives("InvalidVersion", str(tmpdir), {})



from typing import List
import minecraft_launcher_lib
import requests
import pytest
import shutil
import os


def check_argument(command: List[str], argument: str, value: str) -> bool:
    # Checks if a argument has the given value
    try:
        index = command.index(argument)
    except:
        return False
    try:
        return command[index + 1] == value
    except:
        return False


def download_test_version(tmpdir) -> str:
    # Download a random version for test
    version_list = requests.get("https://launchermeta.mojang.com/mc/game/version_manifest.json").json()
    version_id = version_list["versions"][0]["id"]
    r = requests.get(version_list["versions"][0]["url"], stream=True)
    os.makedirs(os.path.join(tmpdir, "versions", version_id))
    with open(os.path.join(tmpdir, "versions", version_id, f"{version_id}.json"), 'wb') as f:
        r.raw.decode_content = True
        shutil.copyfileobj(r.raw, f)
    return version_id


def test_get_minecraft_command_executable_basic(tmpdir):
    version_id = download_test_version(tmpdir)
    options = {}
    options["username"] = "Player"
    options["uuid"] = "Testuuid"
    options["token"] = "testtoken"
    command = minecraft_launcher_lib.command.get_minecraft_command(version_id, str(tmpdir), options)
    assert check_argument(command, "--username", "Player")
    assert check_argument(command, "--uuid", "Testuuid")
    assert check_argument(command, "--accessToken", "testtoken")


def test_get_minecraft_command_executable_path(tmpdir):
    version_id = download_test_version(tmpdir)
    # Test with option
    options = {"executablePath": "test"}
    command = minecraft_launcher_lib.command.get_minecraft_command(version_id, str(tmpdir), options)
    assert command[0] == "test"
    # Test without option
    command = minecraft_launcher_lib.command.get_minecraft_command(version_id, str(tmpdir), {})
    assert command[0] == "java"


def test_get_minecraft_command_jvm_arguments(tmpdir):
    version_id = download_test_version(tmpdir)
    options = {"jvmArguments": ["a", "b"]}
    command = minecraft_launcher_lib.command.get_minecraft_command(version_id, str(tmpdir), options)
    assert command[1] == "a"
    assert command[2] == "b"


def test_get_minecraft_command_game_directory(tmpdir):
    version_id = download_test_version(tmpdir)
    # Test with option
    options = {"gameDirectory": "gamedir"}
    command = minecraft_launcher_lib.command.get_minecraft_command(version_id, str(tmpdir), options)
    assert check_argument(command, "--gameDir", "gamedir")
    # Test without option
    command = minecraft_launcher_lib.command.get_minecraft_command(version_id, str(tmpdir), {})
    assert check_argument(command, "--gameDir", str(tmpdir))


def test_get_minecraft_command_demo(tmpdir):
    version_id = download_test_version(tmpdir)
    # Test with option
    options = {"demo": True}
    command = minecraft_launcher_lib.command.get_minecraft_command(version_id, str(tmpdir), options)
    assert "--demo" in command
    # Test without option
    command = minecraft_launcher_lib.command.get_minecraft_command(version_id, str(tmpdir), {})
    assert "--demo" not in command


def test_get_minecraft_command_custom_resolution(tmpdir):
    version_id = download_test_version(tmpdir)
    # Test default values of width and height
    options = {"customResolution": True}
    command = minecraft_launcher_lib.command.get_minecraft_command(version_id, str(tmpdir), options)
    assert check_argument(command, "--width", "854")
    assert check_argument(command, "--height", "480")
    # Test custom values of width and height
    options = {}
    options["customResolution"] = True
    options["resolutionWidth"] = "1000"
    options["resolutionHeight"] = "500"
    command = minecraft_launcher_lib.command.get_minecraft_command(version_id, str(tmpdir), options)
    assert check_argument(command, "--width", "1000")
    assert check_argument(command, "--height", "500")
    # Test without options
    command = minecraft_launcher_lib.command.get_minecraft_command(version_id, str(tmpdir), {})
    assert "--width" not in command
    assert "--height" not in command


def test_get_minecraft_command_natives_directory(tmpdir):
    version_id = download_test_version(tmpdir)
    # Test with option
    options = {"nativesDirectory": "test"}
    command = minecraft_launcher_lib.command.get_minecraft_command(version_id, str(tmpdir), options)
    assert "-Djava.library.path=test" in command
    # Test without option
    command = minecraft_launcher_lib.command.get_minecraft_command(version_id, str(tmpdir), {})
    default_natives_path = os.path.join(tmpdir, "versions", version_id, "natives")
    assert f"-Djava.library.path={default_natives_path}"


def test_get_minecraft_command_invalid_version(tmpdir):
    # Checks if the VersionNotFound exception raised
    with pytest.raises(minecraft_launcher_lib.exceptions.VersionNotFound):
        minecraft_launcher_lib.command.get_minecraft_command("InvalidVersion", str(tmpdir), {})
